@@ -34,6 +34,7 @@ import os, time
 import datetime
 import hamster
 import hamster.storage
+from hamster import stuff
 import datetime as dt
 import copy
         
@@ -283,34 +284,27 @@ class Storage(hamster.storage.Storage):
 
 
     def __add_fact(self, activity_name, start_time = None, end_time = None):
-        start_time = start_time or datetime.datetime.now()
-        
-        #see if we have description of activity somewhere here (delimited by comma)
-        description = None
-        if activity_name.find(",") > 0:
-            activity_name, description = activity_name.split(",", 1)
-            description = description.strip()
-        
-        if activity_name.lower() in ("bbq_omg", "barbeque_omg"): #this is most essential
-            description = "[ponies = 1], [rainbows = 0]"
+        activity = stuff.parse_activity_input(activity_name)
+        start_time = activity.start_time or start_time or datetime.datetime.now()
+        start_time = start_time.replace(microsecond = 0)
+        end_time = activity.end_time or end_time
+        if end_time:
+            end_time = end_time.replace(microsecond = 0)
+
             
         # now check if maybe there is also a category
         category_id = None
-        if activity_name.find("@") > 0:
-            #at symbol marks category
-            activity_name, category_name = activity_name.split("@", 1)
-            
-            if category_name:
-                category_name = category_name.strip()
-                category_id = self.__get_category_by_name(category_name)
-                if not category_id:
-                    category_id = self.__add_category(category_name)
+        if activity.category_name:
+            category_id = self.__get_category_by_name(activity.category_name)
+            if not category_id:
+                category_id = self.__add_category(activity.category_name)
         
         # try to find activity
-        activity_id = self.__get_activity_by_name(activity_name, category_id)
-
+        activity_id = self.__get_activity_by_name(activity.activity_name,
+                                                  category_id)
         if not activity_id:
-            activity_id = self.__add_activity(activity_name, category_id)
+            activity_id = self.__add_activity(activity.activity_name,
+                                              category_id)
 
 
         # if we are working on +/- current day - check the last_activity
@@ -319,11 +313,13 @@ class Storage(hamster.storage.Storage):
 
             if last_activity and last_activity['start_time'] < start_time:
                 #if this is the same, ongoing activity, then there is no need to create another one
-                if last_activity['activity_id'] == activity_id:
+                if not activity.description \
+                   and last_activity['activity_id'] == activity_id:
                     return last_activity
                 
-                # if it happened in last minute - remove previous entry or stop it
-                if 60 >= (start_time - last_activity['start_time']).seconds >= 0:
+                if not activity.description \
+                   and not last_activity["description"] \
+                   and 60 >= (start_time - last_activity['start_time']).seconds >= 0:
                     self.__remove_fact(last_activity['id'])
                     start_time = last_activity['start_time']
                 else:
@@ -345,14 +341,14 @@ class Storage(hamster.storage.Storage):
                     INSERT INTO facts (activity_id, start_time, end_time, description)
                                VALUES (?, ?, ?, ?)
         """
-        self.execute(insert, (activity_id, start_time, end_time, description))
+        self.execute(insert, (activity_id, start_time, end_time, activity.description))
 
         fact_id = self.fetchone("select max(id) as max_id from facts")['max_id']
         
         return self.__get_fact(fact_id)
 
 
-    def __get_facts(self, date, end_date = None):
+    def __get_facts(self, date, end_date = None, category_id = None):
         query = """
                    SELECT a.id AS id,
                           a.start_time AS start_time,
@@ -362,57 +358,73 @@ class Storage(hamster.storage.Storage):
                           coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id
                      FROM facts a
                 LEFT JOIN activities b ON a.activity_id = b.id
-                LEFT JOIN categories c on b.category_id = c.id
-                    WHERE (date(a.start_time) >= ? and date(a.start_time) <= ?)
-                       OR (date(a.end_time) >= ? and date(a.end_time) <= ?)
-                 ORDER BY a.start_time
+                LEFT JOIN categories c ON b.category_id = c.id
+                    WHERE (a.end_time >= ? OR a.end_time IS NULL) AND a.start_time <= ?
         """
-        end_date = end_date or date
         
-        facts = self.fetchall(query, (_("Unsorted"), date, end_date, date, end_date))
+        if category_id:
+            query += " and b.category_id = %d" % category_id
+
+        query += " ORDER BY a.start_time"
+        end_date = end_date or date
+
+        #FIXME: add preference to set that
+        split_time = dt.time(5, 30)
+        datetime_from = dt.datetime.combine(date, split_time)
+        datetime_to = dt.datetime.combine(end_date, split_time) + dt.timedelta(days = 1)
+        
+        facts = self.fetchall(query, (_("Unsorted"), datetime_from, datetime_to))
         res = []
 
         today = dt.date.today()
-        yesterday = dt.date.today() - dt.timedelta(days=1)
+        yesterday = dt.date.today() - dt.timedelta(days = 1)
         now = dt.datetime.now()
         
-
         # fetch last activity here - we will be looking it up for comparisons
         last_activity = None
-        if date >= today >= end_date or abs(date-today).days < 2 \
-                                   or abs(end_date-today).days < 2:
+        if date >= today >= end_date or abs(date - today).days < 2 \
+                                   or abs(end_date - today).days < 2:
             last_activity = self.__get_last_activity()
 
-        # deal with late-night workers!
-        if not facts and date == today:
-            # no facts today?! let's get last fact of yesterday
-            if last_activity:
-                # last fact has not finished, we think that it is still ongoing
-                f = dict(
-                    id = last_activity["id"],
-                    start_time = dt.datetime.combine(date, dt.time(0,0)),
-                    end_time = now,
-                    description = last_activity["description"],
-                    name = last_activity["name"],
-                    activity_id = last_activity["activity_id"],
-                    category = last_activity["category"],
-                    category_id = last_activity["category_id"],
-                    delta = now - dt.datetime.combine(date, dt.time(0,0))
-                )
-            
-                return [f]
-
-
         for fact in facts:
-            fact_start_date = fact["start_time"].date()
+            # heuristics to assign tasks to proper days
             if fact["end_time"]:
-                fact_end_date = fact["end_time"].date()
+                fact_end_time = fact["end_time"]
+            elif (today - fact["start_time"].date()) <= dt.timedelta(days=1):
+                fact_end_time = now
             else:
-                fact_end_date = None
-                                   
+                fact_end_time = fact["start_time"].replace(hour=23, minute=59)
+
+            fact_start_date = fact["start_time"].date() \
+                - dt.timedelta(1 if fact["start_time"].time() < split_time else 0)
+            fact_end_date = fact_end_time.date() \
+                - dt.timedelta(1 if fact_end_time.time() < split_time else 0)
+            fact_date_span = fact_end_date - fact_start_date
+
+            # check if the task spans across two dates
+            if fact_date_span.days == 1:
+                datetime_split = dt.datetime.combine(fact_end_date, split_time)
+                start_date_duration = datetime_split - fact["start_time"]
+                end_date_duration = fact_end_time - datetime_split
+                if start_date_duration > end_date_duration:
+                    # most of the task was done during the previous day
+                    fact_date = fact_start_date
+                else:
+                    fact_date = fact_end_date
+            else:
+                # either doesn't span or more than 24 hrs tracked
+                # (in which case we give up)
+                fact_date = fact_start_date
+
+            if fact_date < date or fact_date > end_date:
+                # due to spanning we've jumped outside of given period
+                continue
+
             f = dict(
                 id = fact["id"],
                 start_time = fact["start_time"],
+                date = fact_date,
+                delta = fact_end_time - fact["start_time"],
                 end_time = fact["end_time"],
                 description = fact["description"],
                 name = fact["name"],
@@ -420,46 +432,7 @@ class Storage(hamster.storage.Storage):
                 category = fact["category"],
                 category_id = fact["category_id"]
             )
-            
-            if not fact_end_date:
-                if fact_start_date == today and fact["start_time"] < now:
-                    # today, present
-                    f["delta"] = now -  fact["start_time"]
-                elif last_activity and fact_start_date == yesterday and f["id"] == last_activity["id"]:
-                    # last fact and it is in yesterday - split it!
-                    #first yesterday until midnight
-                    f["end_time"] = dt.datetime.combine(today, dt.time(0,0))
-                    f["delta"] = f["end_time"] - f["start_time"]
-                    res.append(f)
-                    
-                    #and now today until now
-                    f = copy.copy(f)
-                    f["start_time"] = dt.datetime.combine(today, dt.time(0, 0))
-                    f["end_time"] = now
-                    f["delta"] = f["end_time"] - f["start_time"]
-                else:
-                    f["delta"] = None
-
-                res.append(f)
-            elif fact_start_date != fact_end_date:
-                # check if maybe we have to split activity in two
-                if date <= fact["start_time"].date()  <= end_date:
-                    start_fact = copy.copy(f)
-                    start_fact["end_time"] = dt.datetime.combine(f["end_time"],
-                                                                 dt.time(0, 0))
-                    start_fact["delta"] = start_fact["end_time"] - start_fact["start_time"]
-                    res.append(start_fact)
-
-                if date <= fact["end_time"].date()  <= end_date:
-                    end_fact = copy.copy(f)
-                    end_fact["start_time"] = dt.datetime.combine(f["end_time"],
-                                                                 dt.time(0, 0))
-                    end_fact["delta"] = end_fact["end_time"] - end_fact["start_time"]
-                    res.append(end_fact)
-            else:
-                #else is we have end date and it is the same date
-                f["delta"] = fact["end_time"] - fact["start_time"]
-                res.append(f)
+            res.append(f)
 
         return res
 
@@ -614,7 +587,7 @@ class Storage(hamster.storage.Storage):
 
     """ Here be dragons (lame connection/cursor wrappers) """
     def get_connection(self):
-        if self.con == None:
+        if self.con is None:
             self.con = sqlite.connect(hamster.HAMSTER_DB, detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
             self.con.row_factory = sqlite.Row
 
@@ -652,11 +625,11 @@ class Storage(hamster.storage.Storage):
         con = self.connection
         cur = con.cursor()
         
-        if type(statement) != list: #we kind of think that we will get list of instructions
+        if isinstance(statement, list) == False: #we kind of think that we will get list of instructions
             statement = [statement]
             params = [params]
             
-        if type(statement) == list:
+        if isinstance(statement, list):
             for i in range(len(statement)):
                 if hamster.trace_sql:
                     print statement[i], params[i]
