@@ -56,7 +56,8 @@ class Dayline(graphics.Area):
         self.move_type = ""
         self.on_time_changed = None #override this with your func to get notified when user changes date
         self.on_more_data = None #supplement with more data func that accepts single date
-        
+        self.in_progress = False
+
         self.range_start = None
         self.in_motion = False
         self.days = []
@@ -70,7 +71,10 @@ class Dayline(graphics.Area):
 
         if event.state & gtk.gdk.BUTTON1_MASK:
             self.call_parent_time_changed()
-    
+
+    def set_in_progress(self, in_progress):
+        self.in_progress = in_progress
+
     def call_parent_time_changed(self):
         #now calculate back from pixels into minutes
         start_time = self.get_value_at_pos(x = self.highlight_start)
@@ -130,14 +134,20 @@ class Dayline(graphics.Area):
         #print x, self.highlight_start, self.highlight_end
         if self.highlight_start != None:
             start_drag = 10 > (self.highlight_start - x) > -1
+
             end_drag = 10 > (x - self.highlight_end) > -1
 
             if start_drag and end_drag:
                 start_drag = abs(x - self.highlight_start) < abs(x - self.highlight_end)
 
             in_between = self.highlight_start <= x <= self.highlight_end
-                
-            
+            scale = True
+
+            if self.in_progress:
+                end_drag = False
+                in_between = False
+                scale = False
+
             if mouse_down and not self.drag_start:
                 self.drag_start = x
                 if start_drag:
@@ -147,7 +157,7 @@ class Dayline(graphics.Area):
                 elif in_between:
                     self.move_type = "move"
                     self.drag_start = x - self.highlight_start
-                else:
+                elif scale:
                     self.move_type = "scale_drag"
                     self.drag_start_time = self.range_start.value
 
@@ -334,8 +344,10 @@ class CustomFactController:
             buf.set_text(fact["description"] or "")
             self.get_widget('description').set_buffer(buf)
 
-            if not end_date and fact["start_time"].date() == dt.date.today():
-                end_date = dt.datetime.now()
+            if not end_date:
+                self.get_widget("in_progress").set_active(True)
+                if (dt.datetime.now() - start_date).days == 0:
+                    end_date = dt.datetime.now()
 
             self.get_widget("save_button").set_label("gtk-save")
             self.window.set_title(_("Update activity"))
@@ -357,10 +369,6 @@ class CustomFactController:
         self.start_date = widgets.DateInput(start_date)
         self.get_widget("start_date_placeholder").add(self.start_date)
         self.start_date.connect("date-entered", self.validate_fields)
-        
-        self.end_date = widgets.DateInput(end_date)
-        self.get_widget("end_date_placeholder").add(self.end_date)
-        self.end_date.connect("date-entered", self.validate_fields)
 
         self.start_time = widgets.TimeInput(start_date)
         self.get_widget("start_time_placeholder").add(self.start_time)
@@ -369,6 +377,7 @@ class CustomFactController:
         self.end_time = widgets.TimeInput(end_date, start_date)
         self.get_widget("end_time_placeholder").add(self.end_time)
         self.end_time.connect("time-entered", self.on_end_time_entered)
+        self.set_end_date_label(end_date)
 
 
         self.set_dropdown()
@@ -386,7 +395,7 @@ class CustomFactController:
         self.start_time.set_time(start_time)
         self.start_date.set_date(start_time)
         self.end_time.set_time(end_time)
-        self.end_date.set_date(end_time)
+        self.set_end_date_label(end_time)
 
         
     def draw_preview(self, date, highlight = None):
@@ -489,9 +498,18 @@ class CustomFactController:
         self.window.show()
 
     def _get_datetime(self, prefix):
-        # adds symbolic time to date in seconds
-        date = getattr(self, prefix + '_date').get_date()
-        time = getattr(self, prefix + '_time').get_time()
+        start_time = self.start_time.get_time()
+        start_date = self.start_date.get_date()
+
+        if prefix == "end":
+            end_time = self.end_time.get_time()
+            end_date = start_date
+            if end_time < start_time:
+                end_date = start_date + dt.timedelta(days=1)
+            self.set_end_date_label(end_date)
+            time, date = end_time, end_date
+        else:
+            time, date = start_time, start_date
         
         if time and date:
             return dt.datetime.combine(date, time.time())
@@ -517,8 +535,6 @@ class CustomFactController:
         # description field is prior to inline description
         return description or inline_description
         
-        
-    
     def on_save_button_clicked(self, button):
         activity = self.get_widget("activity_combo").child.get_text().decode("utf-8")
         
@@ -558,9 +574,12 @@ class CustomFactController:
         return False
         
     def on_in_progress_toggled(self, check):
-        self.end_time.set_sensitive(not check.get_active())
-        self.end_date.set_sensitive(not check.get_active())
+        sensitive = not check.get_active()
+        self.end_time.set_sensitive(sensitive)
+        self.get_widget("end_label").set_sensitive(sensitive)
+        self.get_widget("end_date_label").set_sensitive(sensitive)
         self.validate_fields()
+        self.dayline.set_in_progress(not sensitive)
 
     def on_cancel_clicked(self, button):
         self.close_window()
@@ -575,11 +594,11 @@ class CustomFactController:
         self.validate_fields()
         
     def on_end_time_entered(self, widget):
-        if self.end_time.get_time() < self.start_time.get_time():
-            self.end_date.set_date(self.start_date.get_date() +
-                                                          dt.timedelta(days=1))
         self.validate_fields()
-        
+    
+    def set_end_date_label(self, some_date):
+        self.get_widget("end_date_label").set_text(some_date.strftime("%x"))
+    
     def validate_fields(self, widget = None):
         activity_text = self.get_widget("activity_combo").child.get_text()
         start_time = self._get_datetime("start")
@@ -588,14 +607,18 @@ class CustomFactController:
         if self.get_widget("in_progress").get_active():
             end_time = dt.datetime.now()
 
+        # if we are too far, just roll back for one day
+        if ((end_time - start_time).days > 0): 
+            end_time -= dt.timedelta(days=1)
+            self.update_time(start_time, end_time)
+
         if start_time:
             self.draw_preview(start_time.date(), [start_time, end_time])
 
+        # if end time is not in proper distance, do the brutal +30 minutes reset
         if (end_time < start_time or (end_time - start_time).days > 0):
             end_time = start_time + dt.timedelta(minutes = 30)
             self.update_time(start_time, end_time)
-            self.validate_fields()
-            return
 
         looks_good = False
         if activity_text != "" and start_time and end_time and \
@@ -610,7 +633,6 @@ class CustomFactController:
               and event_key.state & gtk.gdk.CONTROL_MASK)):
             
             if self.start_date.popup.get_property("visible") or \
-               self.end_date.popup.get_property("visible") or \
                self.start_time.popup.get_property("visible") or \
                self.end_time.popup.get_property("visible"):
                 return False
