@@ -47,8 +47,8 @@ class Storage(object):
         self.dispatch('day_updated', fact['start_time'])
         return result
 
-    def get_facts(self, date, end_date = None):
-        return self.__get_facts(date, end_date)
+    def get_facts(self, date, end_date = None, category_id = None):
+        return self.__get_facts(date, end_date, category_id)
 
     def get_popular_categories(self):
         return self.__get_popular_categories()
@@ -119,4 +119,9 @@ class Storage(object):
 
     def get_category_list(self):
         return self.__get_category_list()
-        
+
+    def get_category_by_name(self, category):
+        return self.__get_category_by_name(category)
+
+    def get_activity_by_name(self, activity, category_id):
+        return self.__get_activity_by_name(activity, category_id)
